#!/usr/bin/env python
# Python
import os
import sys
import time

# Insights
from redhat_access_insights import constants as insights_constants, parse_config_file
from redhat_access_insights.auto_config import try_auto_configuration

# Coordinator
from insights_coordinator_rhosp.constants import RhospCoordinatorConstants
from insights_coordinator_rhosp.config import RhospCoordinatorConfig
from insights_coordinator_rhosp.parser import RhospCoordinatorParser
from insights_coordinator_rhosp.utilities import RhospCoordinatorUtilities
from insights_coordinator_rhosp.director import RhospCoordinatorDirector

from insights_coordinator_rhosp.insights_coordinator_infra.utils.cron import CoordinatorCron as RhospCoordinatorCron
from insights_coordinator_rhosp.insights_coordinator_infra.transport.ssh import CoordinatorSSH as RhospCoordinatorSSH
from insights_coordinator_rhosp.insights_coordinator_infra.logger import CoordinatorLogger as RhospCoordinatorLogger
from insights_coordinator_rhosp.insights_coordinator_infra.utils.language import CoordinatorLanguage as RhospCoordinatorLanguage
from insights_coordinator_rhosp.insights_coordinator_infra.utils.versions import CoordinatorVersions as RhospCoordinatorVersions

"""
Main entry point for the RHOSP Coordinator
"""


def main():

    # Setup the Language Handler
    rhosp_language = RhospCoordinatorLanguage()

    # Run the Coordinator as root
    if os.geteuid() is not 0:
        sys.exit(rhosp_language.get_language("Insights Coordinator must be run as root."))

    # Setup the Coordinator Constants
    rhosp_constants = RhospCoordinatorConstants()

    # Setup the Coordinator Configuration File
    rhosp_config = RhospCoordinatorConfig(constants=rhosp_constants,
                                          language=rhosp_language)

    # Configure the Insights Uploader Connection
    uploader_conf = parse_config_file(insights_constants.default_conf_file)
    try_auto_configuration(uploader_conf)

    # Parse the options
    options_parser = RhospCoordinatorParser(constants=rhosp_constants,
                                            config=rhosp_config,
                                            language=rhosp_language)
    options = options_parser.parse_options()
    options.uploader_conf = uploader_conf
    if options.interactive:
        options.verbose = True

    # Setup the Logger
    rhosp_logger = RhospCoordinatorLogger(constants=rhosp_constants,
                                          config=rhosp_config,
                                          language=rhosp_language,
                                          verbose=options.verbose)
    rhosp_config.setup_logger(rhosp_logger)
    options_parser.setup_logger(rhosp_logger)

    # Print configuration and options
    rhosp_config.print_config()
    options_parser.print_options()

    # Display the current coordinator version and exit if --version flag is thrown
    if options.version:
        sys.exit(str(rhosp_constants.version))

    # Exit if a proper uploader configuration file is not found
    # TODO: If a valid uploader_conf was not found BUT --archive-only was thrown as a flag, should we continue?
    # Does this imply that the Insights client is not installed?
    if not uploader_conf:
        sys.exit(rhosp_language.get_language("Error configuring Insights Client. "
                                             "Please ensure you have Insights installed and it is up to date."))

    # Schedule the Coordinator Cron Job
    cron = RhospCoordinatorCron(constants=rhosp_constants,
                                config=rhosp_config,
                                logger=rhosp_logger,
                                language=rhosp_language)
    rhosp_config.setup_cron(cron)

    # Run the configuration utility if the command line flag is present, then exit
    if options.configure:
        rhosp_config.run_config()
        sys.exit(rhosp_language.get_language("RHOSP Insights Coordinator Configuration utility completed."))

    # Both enable_cron and disable_cron cant both be flagged or passed in as options
    # If both are passed in then exit with the appropriate message
    if options.enable_cron and options.disable_cron:
        sys.exit(rhosp_language.get_language("Invalid flag combination. "
                                             "You can either enable the Coordinator cron job "
                                             "or disable the Coordinator cron job but not both."))

    # Enable the Cron
    if options.enable_cron:
        cron.setup_cron()
        rhosp_config.write_config_var('enable_cron', True)
        sys.exit(rhosp_language.get_language("Cron Job enabled."))
    if rhosp_config.get_config_var('enable_cron', True):
        if not cron.cron_exists():
            cron.setup_cron()

    # Disable the Cron
    if options.disable_cron:
        cron.disable_cron()
        rhosp_config.write_config_var('enable_cron', False)
        sys.exit(rhosp_language.get_language("Cron Job disabled."))
    if not rhosp_config.get_config_var('enable_cron', True):
        if cron.cron_exists():
            cron.disable_cron()

    # RHOSP Utilities
    rhosp_utilities = RhospCoordinatorUtilities(constants=rhosp_constants,
                                                logger=rhosp_logger,
                                                uploader_conf=uploader_conf,
                                                config=rhosp_config,
                                                language=rhosp_language)
    rhosp_config.setup_utilities(rhosp_utilities)

    # Setup SSH
    rhosp_ssh_client = RhospCoordinatorSSH(constants=rhosp_constants,
                                           logger=rhosp_logger,
                                           config=rhosp_config,
                                           language=rhosp_language)

    # Instantiate the RHOSP Director
    rhosp_director = RhospCoordinatorDirector(rhosp_constants, rhosp_logger,
                                              rhosp_ssh_client, rhosp_utilities,
                                              rhosp_config, rhosp_language)

    # Instantiate the versions handler
    rhosp_versions = RhospCoordinatorVersions(coordinator_constants=rhosp_constants,
                                              insights_constants=insights_constants,
                                              language=rhosp_language,
                                              logger=rhosp_logger,
                                              utilities=rhosp_utilities)

    # Import the RHOSP Coordinator Module
    import insights_coordinator_rhosp.osp as platform_module
    if not platform_module:
        rhosp_logger.vlog(rhosp_language.get_language("RHOSP coordinator platform was not found. "
                                                      "Please reinstall insights-coordinator-rhosp."),
                          level="FATAL")
        sys.exit(1)

    # Run the Coordinator
    platform_module.run(options=options,
                        constants=rhosp_constants,
                        utilities=rhosp_utilities,
                        logger=rhosp_logger,
                        uploader_conf=uploader_conf,
                        config=rhosp_config,
                        language=rhosp_language,
                        director=rhosp_director,
                        ssh_client=rhosp_ssh_client,
                        versions=rhosp_versions)

"""
MAIN EXECUTION
"""
if __name__ == "__main__":
    main()
