#!/usr/bin/env python

import sys
import os
import novaclient

from novaclient.client import Client as NovaClient
from neutronclient.v2_0 import client as NeutronClient

from insights_coordinator_rhosp.insights_coordinator_infra.api import CoordinatorApi


class RhospCoordinatorApi(CoordinatorApi):

    # Containers
    stack_client = None
    overcloud_client = None
    neutron_overcloud_client = None
    neutron_stack_client = None
    credentials = None
    hosts = None
    services = None
    overcloud_networks = None
    stack_networks = None

    # Constructor
    def __init__(self, **kwargs):
        super(RhospCoordinatorApi, self).__init__(**kwargs)
        self.credentials = kwargs['credentials']
        self.setup_api()

    # Get deployment name from overcloudrc environment variable
    def get_deployment_name(self):
        self.credentials.setup_env('overcloudrc_env')
        return os.environ['OS_CLOUDNAME']

    # Get the nova client API version
    def get_nova_client_version(self):
        return novaclient.__version__

    # Get the nova client API
    def get_api(self, the_client="stack_client"):
        if not getattr(self, the_client):
            self.setup_api()
        return getattr(self, the_client)

    # Setup the OSP client APIs
    def setup_api(self):
        try:
            self.stack_client = NovaClient(**self.credentials.get_stack_credentials())
            self.overcloud_client = NovaClient(**self.credentials.get_overcloud_credentials())
            self.neutron_overcloud_client = NeutronClient.Client(**self.credentials.get_neutron_overcloud_credentials())
            self.neutron_stack_client = NeutronClient.Client(**self.credentials.get_neutron_stack_credentials())
        except:
            sys.exit("There was an issue obtaining proper credentials for the OSP Environment. Please ensure you have proper stackrc and overcloudrc files configured.")
        if not self.stack_client or not self.overcloud_client or not self.neutron_overcloud_client or not self.neutron_stack_client:
            sys.exit("There was an issue obtaining proper credentials for the OSP Environment. Please ensure you have proper stackrc and overcloudrc files configured.")
        return True

    # Get the nova list / servers
    def get_hosts(self):
        if not self.hosts:
            self.setup_hosts()
        return self.hosts

    # Get the nova service types
    def get_services(self):
        if not self.services:
            self.setup_services()
        return self.services

    # Get ALL Neutron networks and clients
    def get_networks(self):
        if not self.overcloud_networks or not self.stack_networks:
            self.setup_networks()
        return {'overcloud': self.overcloud_networks,
                'stack': self.stack_networks}

    # Get the neutron overcloud network information
    def get_overcloud_networks(self):
        if not self.overcloud_networks:
            self.setup_overcloud_networks()
        return self.overcloud_networks

    # Get the neutron stack network information
    def get_stack_networks(self):
        if not self.stack_networks:
            self.setup_stack_networks()
        return self.stack_networks

    # Setup the nova list / servers
    def setup_hosts(self):
        nova_client = self.get_api('stack_client')
        self.hosts = nova_client.servers.list()
        self.hosts = self.clean_hosts(self.hosts)
        return self.hosts

    # Setup the nova services list
    def setup_services(self):
        nova_client = self.get_api('overcloud_client')
        self.services = nova_client.services.list()
        self.services = self.clean_services(self.services)
        return self.services

    # Setup the network list from the neutron client
    def setup_networks(self):

        # Get the Neutron Overcloud Client and Networks
        self.setup_overcloud_networks()

        # Get the Neutron Stack Client and Networks
        self.setup_stack_networks()

        return {'overcloud': self.overcloud_networks,
                'stack': self.stack_networks}

    # Setup the overcloud networks and client
    def setup_overcloud_networks(self):
        neutron_overcloud_client = self.get_api('neutron_overcloud_client')
        self.overcloud_networks = neutron_overcloud_client.list_networks()['networks']
        self.overcloud_networks = self.clean_networks(self.overcloud_networks)
        return self.overcloud_networks

    # Setup the stack networks and client
    def setup_stack_networks(self):
        neutron_stack_client = self.get_api('neutron_stack_client')
        self.stack_networks = neutron_stack_client.list_networks()['networks']
        self.stack_networks = self.clean_networks(self.stack_networks)
        return self.stack_networks

    # Clean the host list
    def clean_hosts(self, host_list):
        cleaned_hosts = []
        for host in host_list:
            cleaned_host = {}
            cleaned_host['id'] = host.id
            cleaned_host['name'] = host.name
            cleaned_host['status'] = host.status
            cleaned_host['networks'] = host.networks['ctlplane'][0]
            cleaned_hosts.append(cleaned_host)
        return cleaned_hosts

    # Clean the service list
    def clean_services(self, service_list):
        cleaned_services = []
        for service in service_list:
            cleaned_service = {}
            cleaned_service['name'] = service.host.split('.')[0]  # SPLIT on '.' because services return the FQDN
            cleaned_service['fqdn'] = service.host
            cleaned_service['service'] = service.binary.replace('nova-', '').lower()
            cleaned_service['state'] = service.state
            cleaned_service['status'] = service.status
            cleaned_service['zone'] = service.zone
            cleaned_services.append(cleaned_service)
        return cleaned_services

    # Clean up the networks retrieve from neutron sdks
    # Placeholder for now, don't think we really need to clean them up at the moment
    def clean_networks(self, networks):
        return networks

    # Print the host list
    def print_hosts(self, specific_list):
        hosts = specific_list if specific_list else self.get_hosts()
        relevant_keys = specific_list[0].keys() if specific_list else ['id', 'name', 'status', 'networks']
        self.logger.vlog(("%-40s" * len(relevant_keys)) % tuple(relevant_keys))
        self.logger.vlog("------------------------------------------------------")
        for host in hosts:
            host_to_print = []
            for key in relevant_keys:
                host_to_print.append(str(host[key]))
            self.logger.vlog(("%-40s" * len(relevant_keys)) % tuple(host_to_print))
        self.logger.vlog("=======================================================")

    # Print the service list
    def print_services(self, specific_list):
        services = specific_list if specific_list else self.get_services()
        relevant_keys = specific_list[0].keys() if specific_list else ['name', 'service', 'state', 'status', 'zone']
        self.logger.vlog(("%-40s" * len(relevant_keys)) % tuple(relevant_keys))
        self.logger.vlog("------------------------------------------------------")
        for service in services:
            service_to_print = []
            for key in relevant_keys:
                    service_to_print.append(str(service[key]))
            self.logger.vlog(("%-40s" * len(relevant_keys)) % tuple(service_to_print))
        self.logger.vlog("=======================================================")

    # Get all relevant information from the RHOSP Director API
    def get_director_data_from_api(self, options=None):
        director_information = {}

        # Right now we return an empty object
        # TODO: Research what possible information we can obtain from the RHOSP Python SDKs

        return director_information
