#!/usr/bin/env python

import glob
import json
import re

from insights_coordinator_rhosp.credentials import RhospCoordinatorCredentials
from insights_coordinator_rhosp.api import RhospCoordinatorApi
from insights_coordinator_rhosp.node import RhospCoordinatorNode

from nova import version as nova_version


class RhospCoordinatorDirector():

    # Containers
    constants = None
    logger = None
    utilities = None
    api = None
    node = None
    ssh_client = None
    config = None
    language = None
    node_list = None
    service_list = None
    blacklist = False

    # Constructor
    def __init__(self, rhosp_constants, rhosp_logger, ssh_client, rhosp_utilities, rhosp_config, rhosp_language):
        self.constants = rhosp_constants
        self.config = rhosp_config
        self.logger = rhosp_logger
        self.utilities = rhosp_utilities
        self.language = rhosp_language
        self.credentials = RhospCoordinatorCredentials(constants=self.constants,
                                                       config=self.config,
                                                       logger=self.logger)

        self.api = RhospCoordinatorApi(constants=self.constants,
                                       config=self.config,
                                       logger=self.logger,
                                       credentials=self.credentials,
                                       language=self.language)
        self.ssh_client = ssh_client
        self.node = RhospCoordinatorNode(constants=self.constants,
                                         config=self.config,
                                         logger=self.logger,
                                         utilities=self.utilities,
                                         ssh_client=self.ssh_client,
                                         director=self,
                                         language=self.language)

    # Setup the Blacklist Containers for Nodes
    def setup_blacklist(self, blacklist):
        self.blacklist = blacklist

    # Test the RHOSP API
    # Returns boolean status connection
    def test_api(self):
        return True if self.api.get_api() else False

    # Install the latest insights RPM on the Director
    # This RPM will be used to distribute to other nodes
    def install_director(self, options=None):
        self.logger.vlog(self.language.get_language("Director does not contain Insights installation. "
                                                    "Attempting to install Insights on director..."))

        install_cmd = self.utilities.get_command('install_director_auto')
        self.logger.vlog(self.language.get_language("Installing Insights on the director..."))
        self.utilities.run_command(install_cmd)

    # Check if Insights is on the Director
    def check_director_installation(self):
        insights_installations = glob.glob(self.config.get_config_var('local_director_installation'))
        if insights_installations:
            self.logger.vlog(self.language.get_language("Check for Insights installation on the director passed."))
            return True
        else:
            self.logger.vlog(self.language.get_language("Check for Insights installation on the director failed."))
            return False

    # Get the instance of the Node Host
    def get_node(self):
        return self.node

    # Get nodes from the director
    def get_node_list(self):
        if not self.node_list:
            node_list = self.api.get_hosts()
            node_list = self.test_node_list(node_list)
            self.node_list = node_list

            self.logger.vlog(self.language.get_language("Gathering node list from director..."))
            self.logger.vlog(self.language.get_language("Found nodes:"))
            self.logger.vlog(node_list)

        return self.node_list

    # Get service types for the nodes
    def get_service_list(self):
        if not self.service_list:
            service_list = self.api.get_services()
            self.service_list = service_list

            self.logger.vlog(self.language.get_language('Gathering service list from director...'))
            self.logger.vlog(self.language.get_language('Found services:'))
            self.logger.vlog(service_list)

        return self.service_list

    # Test a list of nodes
    def test_node_list(self, node_list):
        tested_list = []
        if node_list:
            director_system_id = self.get_director_system_id()
            for node in node_list:
                parsed_node = {'ip': node['networks'],
                               'id': self.node.generate_machine_id(director_system_id, node['id']),  # Generate a unique type 5 UUID based off of the director UUID and the node UUID
                               'name': node['name'],
                               'status': node['status'],
                               'blacklisted': True if (self.blacklist and len(self.blacklist) > 0 and len([keyword for keyword in self.blacklist if re.match(keyword, node['name'])]) > 0) else False}
                parsed_node['is_alive'] = self.node.test_alive(node['networks'])
                parsed_node['can_ssh'] = self.node.test_ssh(node['networks'])
                tested_list.append(parsed_node)
        return tested_list

    # Print the node list
    def print_node_list(self, node_list):
        if node_list:
            self.api.print_hosts(node_list)

    # Print the service list
    def print_service_list(self, service_list):
        if service_list:
            self.api.print_services(service_list)

    # Collect information for the director from the API
    def collect_director_information(self, options=None):
        if not self.utilities.get_tmpdir():
            self.utilities.setup_tmpdir()
        director_information = {}

        # Get all metadata from API
        self.logger.vlog(self.language.get_language("Using the RHOSP Director APIs to collect director information..."))
        self.logger.vlog(self.language.get_language("Using Director options:"))
        self.logger.vlog(options)
        director_information = self.api.get_director_data_from_api(options)

        # Get RHOSP Nova Client API Version
        nova_client_api_version = self.get_rhosp_nova_client_api_version()
        self.logger.vlog(self.language.get_language("Getting RHOSP Director Nova Client API Version..."))
        self.logger.vlog(nova_client_api_version)
        director_information["nova_client_api_version"] = nova_client_api_version

        # Get Director OS Version
        os_version = self.get_rhosp_os_version()
        rhel_version = self.get_rhel_os_version()
        self.logger.vlog(self.language.get_language("Getting RHOSP Director OS Version..."))
        self.logger.vlog(os_version)
        director_information["rhosp_version"] = os_version
        director_information["rhel_version"] = rhel_version

        # Get Insights Version
        insights_version = self.get_insights_version()
        self.logger.vlog(self.language.get_language("Getting Insights Version..."))
        self.logger.vlog(insights_version)
        director_information["insights_version"] = insights_version

        # Get Coordinator Version
        coordinator_version = self.get_coordinator_version()
        self.logger.vlog(self.language.get_language("Getting Coordinator Version..."))
        self.logger.vlog(coordinator_version)
        director_information["coordinator_version"] = coordinator_version

        # Setup the system relationships for the director and nodes
        director_information["systems"] = self.get_systems()

        # Setup the system/cluster ID
        director_information["system_id"] = self.utilities.get_cluster_id()

        # Get the director name
        director_information["display_name"] = self.get_deployment_name()

        # Add the product information
        # This allows the engine to intelligently decide what to do for the upload
        director_information["product"] = self.get_product_code()

        # Get the neutron network information
        self.logger.vlog(self.language.get_language('Getting Neutron networks...'))
        neutron_overcloud_networks = self.api.get_overcloud_networks()
        director_information['overcloud_networks'] = neutron_overcloud_networks
        director_information['total_overcloud_networks'] = len(neutron_overcloud_networks)
        neutron_stack_networks = self.api.get_stack_networks()
        director_information['stack_networks'] = neutron_stack_networks
        director_information['total_stack_networks'] = len(neutron_stack_networks)

        # Write the log
        log_information_string = json.dumps(director_information, default=self.utilities.serialize_default)
        self.logger.vlog(self.language.get_language("Writing director information to temporary directory..."))
        self.utilities.write_file(log_information_string, self.constants.director_metadata_filename)

    # Get system relationships for nodes and director
    def get_systems(self):

        # Systems container
        systems = []

        # Get the director system ID
        # Used for linking nodes to the director
        director_system_id = self.get_director_system_id()

        # Create the systems metadata and relationships
        for host in self.get_node_list():

            # Determine the node service type
            # Hacky, re-do this logic somehow
            # Maybe key/pairs for faster execution
            node_service_type = 'compute'
            for service_type in self.get_service_list():
                if service_type['name'] == host['name']:
                        if service_type['service'] != 'compute':
                            node_service_type = 'controller'
                        break

            # Setup the node object
            thehost = {}
            thehost['type'] = node_service_type.lower()
            thehost['product'] = self.constants.product_code
            thehost['system_id'] = host['id']  # This has already been generated based off of the director UUID and node UUID
            thehost['display_name'] = host['name']
            thehost['status'] = host['status']
            thehost['ip'] = host['ip']
            thehost['links'] = [{'type': self.get_director_type(), 'system_id': director_system_id}]
            thehost['blacklisted'] = host['blacklisted']
            systems.append(thehost)

        # Setup the director metadata
        director = {}
        director['type'] = self.get_director_type()
        director['product'] = self.constants.product_code
        director['system_id'] = director_system_id
        director['display_name'] = self.get_director_name()
        director['links'] = []
        director['blacklisted'] = False
        systems.append(director)

        # Return all systems in the stack
        return systems

    # Get the alias/type for the director
    def get_director_type(self):
        return 'director'

    # Get the director name / ID
    def get_director_name(self):
        return self.utilities.run_command(self.utilities.get_command('get_director_name')).rstrip()

    # Get the director system ID
    def get_director_system_id(self):
        return self.utilities.run_command(self.utilities.get_command('get_director_system_id')).rstrip()

    # Get the deployment name
    def get_deployment_name(self):
        deployment_name = self.api.get_deployment_name()
        if deployment_name == 'overcloud':
            deployment_name = self.get_director_name()
        return deployment_name

    # Get the RHOSP Director Nova Client API Version for Metadata
    def get_rhosp_nova_client_api_version(self):
        return self.api.get_nova_client_version()

    # Get the RHOSP OS Version for Metadata
    def get_rhosp_os_version(self):
        return nova_version.version_string()

    # Get the RHEL OS Version for Metadata
    def get_rhel_os_version(self):
        return self.utilities.run_command(self.utilities.get_command('get_os_version')).rstrip()

    # Get the Insights Version for Metadata
    def get_insights_version(self):
        return self.utilities.run_command(self.utilities.get_command('get_insights_version')).rstrip()

    # Get the Coordinator Version for Metadata
    def get_coordinator_version(self):
        return self.constants.version

    # Get the product code for this coordinator
    def get_product_code(self):
        return self.constants.product_code

    # Run Insights on the Director
    def run_insights_on_director(self):

        self.logger.vlog(self.language.get_language('Running Insights on director'))
        command_to_run = self.utilities.get_command('run_insights_on_director')
        stdout = self.utilities.run_command(command_to_run)
        self.utilities.write_archive(stdout, 'insights-coordinator-director')
