#!/usr/bin/env python
import sys
import ConfigParser
import getpass
import glob


class CoordinatorConfig(object):

    # Containers
    constants = None
    config = None
    temp_config = None
    logger = None
    utilities = None
    language = None
    cron = None

    # Constructor
    def __init__(self, **kwargs):
        self.constants = kwargs['constants']
        self.language = kwargs['language']

    # Setup the Utilities
    def setup_utilities(self, utilities):
        self.utilities = utilities

    # Setup the Logger
    def setup_logger(self, logger):
        self.logger = logger

    # Setup the CRON Instance
    def setup_cron(self, cron):
        self.cron = cron

    # Get type of configuration variable
    def get_var_type(self, config_var):
        for typ in self.constants.config_data_type_map:
            if config_var in self.constants.config_data_type_map[typ]:
                return typ
        return 'string'

    # Get configuration variables from the coordinator configuration file
    # This will favor a temporary configuration value over an actual configuration value
    # This allows for the ability to override configuration values during runtime and as options
    def get_config_var(self, config_var, override=False):

        # Get the actual configuration
        config = self.get_config()

        # Get the temporary configuration, as would be setup by any overriden command line parameters/flags
        temp_config = self.get_temp_config()

        # Create holders for the config and temp config values
        config_value = None
        var_type = self.get_var_type(config_var)
        temp_config_value = temp_config[config_var] if config_var in temp_config else None

        # Try to get the configuration value from the file
        try:
            config_value = config.get(self.constants.app_name, config_var)
        except(ConfigParser.NoSectionError, ConfigParser.NoOptionError, ValueError, AttributeError):
            pass

        # Default behavior is:
        # 1) Try to obtain the config value
        # 2) Obtain the temp config value
        # 3) If they are not equal, then use the temporary configuration value
        # This implies that we have overriden the usual config value with a flag
        # "override" allows us to bypass the default functionality, and only use the actual config value
        if override is True and config_value:
            return self.parse_config_var_type(config_value, var_type)
        elif override is True:
            return config_value

        # Return the temporary configuration value if they are not equal
        # This implies that a command line flag has been thrown
        if config_value or temp_config_value:
            if temp_config_value != config_value and temp_config_value is not None:
                return self.parse_config_var_type(temp_config_value, var_type)
            return self.parse_config_var_type(config_value, var_type)
        else:
            if not hasattr(self.constants, config_var):
                return None
            return self.parse_config_var_type(getattr(self.constants, config_var), var_type)

    # Parse the configuration variable type
    # Ensures we get back bool, string etc
    def parse_config_var_type(self, config_value, var_type='string'):
        if var_type == 'boolean':
            return True if (config_value is True or config_value in ['1', 'yes', 'true', 'True', 'on']) else False
        else:
            return config_value

    # Get the entire configuration object
    def get_config(self):
        if not self.config:
            self.setup_config()
        return self.config

    # Get a temporary configuration
    # This is a copy of the current configuration but allows
    # the user to override configuration values for this one execution
    # this allows an easier method to pass in temporary
    # options that will override existing configuration values
    def get_temp_config(self):
        if not self.temp_config:
            self.setup_config()
        return self.temp_config

    # Validate configuration in .conf file for type check and file
    # exist/permission check
    def validate_config(self):
        for key, value in self.constants.config_data_type_map.items():
            for field in value:
                try:
                    if key == 'int':
                        self.config.getint(self.constants.app_name, field)
                    elif key == 'boolean':
                        self.config.getboolean(self.constants.app_name, field)
                    elif key == 'path':
                        glob.glob(self.config.get(self.constants.app_name, field))[0]
                except ConfigParser.NoOptionError:
                    pass
                except ValueError:
                    sys.exit(self.language.get_language("Invalid value '%s' for '%s' in '%s'. Value should be of type %s.") % (self.config.get(self.constants.app_name, field), field, self.constants.conf_file, key))
                except IndexError:
                    sys.exit(self.language.get_language("File path '%s' for '%s' in '%s' does not exist." % (self.config.get(self.constants.app_name, field), field, self.constants.conf_file)))

    # Setup the configuration object
    # This will read the configuration file and convert it
    def setup_config(self):
        self.config = ConfigParser.RawConfigParser()
        self.temp_config = {}
        self.config.read(self.constants.conf_file)
        if not self.config.has_section(self.constants.app_name):
            self.config.add_section(self.constants.app_name)
        self.validate_config()

    # Write a single configuration variable to the configuration file
    def write_config_var(self, config_var, config_val):
        config = self.get_config()
        config.set(self.constants.app_name, config_var, config_val)
        with open(self.constants.conf_file, 'wb') as configfile:
            config.write(configfile)
        configfile.close()

    # Write the entire configuration file
    # { key1:val1, key2:val2 }
    def write_config_file(self, configuration_values):
        config = self.get_config()
        for key, value in configuration_values.iteritems():
            config.set(self.constants.app_name, key, value)
        with open(self.constants.conf_file, 'wb') as configfile:
            config.write(configfile)
        configfile.close()

    def set_temp_config_var(self, temp_config_var, temp_config_val):
        temp_config = self.get_temp_config()
        temp_config[temp_config_var] = temp_config_val

    def print_config(self):
        # Print the needful
        self.logger.vlog(self.language.get_language('File configuration:'))
        file_config = self.get_config()
        file_sections = file_config.sections()
        for section in file_sections:
            self.logger.vlog('Section: %s' % (section))
            self.logger.vlog(file_config.items(section))
        self.logger.vlog(self.language.get_language('Temporary configuration:'))
        self.logger.vlog(self.get_temp_config())
