#!/usr/bin/env python
import subprocess
import os
import shutil
import tempfile
import uuid
from datetime import datetime
from datetime import date

from redhat_access_insights import connection
from redhat_access_insights.connection import InsightsConnection
from redhat_access_insights.collection_rules import InsightsConfig
from ..commands import CoordinatorCommands
from .cleaner import CoordinatorCleaner


class CoordinatorUtilities(object):

    # Containers
    constants = None
    logger = None
    tmpdir = None
    uploader_conf = None
    commands = None
    config = None
    language = None
    cleaner = None

    # Constructor
    def __init__(self, **kwargs):
        self.constants = kwargs['constants']
        self.logger = kwargs['logger']
        self.uploader_conf = kwargs['uploader_conf']
        self.config = kwargs['config']
        self.language = kwargs['language']
        self.commands = CoordinatorCommands()
        self.cleaner = CoordinatorCleaner(constants=self.constants,
                                          logger=self.logger,
                                          config=self.config,
                                          utilities=self,
                                          language=self.language)

    #######################
    # SHEXEC #
    #####################

    # Takes a raw command to run
    # TODO: Have this return string results, or boolean results
    def run_command(self, cmd, return_type="String"):
        output = None
        if cmd:
            try:
                output = subprocess.Popen([cmd], shell=True,
                                          stdout=subprocess.PIPE).communicate()[0]
            except subprocess.CalledProcessError as e:
                self.logger.vlog(e.output, "ERROR")
                raise e
        return output

    # Get a command from the CoordinatorCommands instance
    def get_command(self, cmd):
        if not self.commands:
            self.commands = CoordinatorCommands()
        if hasattr(self.commands, cmd):
            return getattr(self.commands, cmd)
        else:
            return None

    #########################
    # TEMPS #
    #######################

    # Setup the temporary directory to store the archives
    def setup_tmpdir(self):
        tdir = tempfile.mkdtemp(prefix=self.constants.default_tmpdir_prefix)
        self.tmpdir = tdir
        self.logger.vlog(self.language.get_language("Setting up temporary directory %s") % (tdir))
        return tdir

    # Return the current temporary working directory
    def get_tmpdir(self):
        if not self.tmpdir:
            self.setup_tmpdir()
        return self.tmpdir

    # Clean the archives
    def clean_tmps(self):
        self.logger.vlog(self.language.get_language("Cleaning temporary files..."))
        if self.get_tmpdir():
            self.logger.vlog(self.language.get_language("Removing directory %s") % (self.get_tmpdir()))
            shutil.rmtree(self.get_tmpdir())
        if self.get_tmpgz():
            self.logger.vlog(self.language.get_language("Removing temporary file %s") % (self.get_tmpgz()))
            os.remove(self.get_tmpgz())

    #######################
    # GZS #
    #####################
    # Return the current temporary GZ
    def get_tmpgz(self):
        return "%s.tar.gz" % (self.get_tmpdir())

    # Remove an individual archive gz
    def remove_gz(self, archivename):
        self.logger.vlog(self.language.get_language("Removing file %s") % (archivename))
        os.remove(archivename)

    # Untar an individual archive gz
    def untar_gz(self, archivename):
        untar_command = self.get_command('untar_gz') % (self.get_tmpdir(), archivename)
        untar_execution = int(self.run_command(untar_command))
        if untar_execution == 0:
            self.logger.vlog(self.language.get_language("Successfully untarred %s in %s") %
                             (archivename, self.get_tmpdir()))
            return True
        else:
            self.logger.vlog(self.language.get_language("There was a problem untarring %s in %s") %
                             (archivename, self.get_tmpdir()))
            return False

    # Retar a GZ
    def retar_gz(self, foldername):
        retar_command = self.get_command('retar_gz') % (foldername, foldername, foldername)
        retar_execution = int(self.run_command(retar_command))
        if retar_execution == 0:
            self.logger.vlog(self.language.get_language("Sucessfully retarred %s") % foldername)
            return True
        else:
            self.logger.vlog(self.language.get_language("There was a problem retarring %s") % foldername)
            return False

    ###########################
    # ARCHIVES #
    #########################

    # Compress an individual archive using stdout from a hypervisor/node
    def write_archive(self, stdout, archive_prefix=None, untar=True, remove_file=True, retar=True):
        tmpdir = self.get_tmpdir()
        if not stdout or stdout == "":
            return False
        if not archive_prefix:
            archive_prefix = self.constants.default_archive_prefix
        tfile = tempfile.NamedTemporaryFile(dir=tmpdir, prefix=archive_prefix, delete=False)
        tfile.write(stdout)
        tfile.flush()
        tfile.close()
        shutil.move(tfile.name, tfile.name + '.tar')
        self.logger.vlog(self.language.get_language("Wrote archive to %s") % (tfile.name + '.tar'))

    # Write a file...
    def write_file(self, stdout, filename):
        tmpdir = self.get_tmpdir()
        if not stdout or stdout == "":
            return False
        f = open(os.path.join(tmpdir, filename), 'wb')
        f.write(stdout)
        f.close()

    # Read a file...
    def read_file(self, filename):
        tmpdir = self.get_tmpdir()
        f = open(os.path.join(tmpdir, filename), 'rb')
        contents = f.read()
        f.close()
        return contents

    # Compress archives to be sent to Red Hat
    def compress_archives(self):
        tmpdir = self.get_tmpdir()
        tmpgz = self.get_tmpgz()
        soscleaner = self.config.get_config_var('soscleaner')

        # untar archives received from insights stdout
        self.run_command(self.get_command('untar_everything') % (tmpdir))

        # remove archives received from insights stdout
        self.run_command(self.get_command('remove_all_tars') % (tmpdir))

        # Clean all of the folders before we tar them up and remove them IF the SOSCleaner is enabled
        if soscleaner is True:
            # Clean the temporary directory (Creating soscleaner.tar.gz)
            self.cleaner.clean_archive(self.get_tmpdir())

            # Remove all of the old unobfuscated directories
            self.run_command(self.get_command('remove_all_dirs') % (tmpdir))

            # Remove the old unobfuscated metadata.json file
            os.remove(os.path.join(self.get_tmpdir(), 'metadata.json'))

            # Untar the soscleaner.tar.gz to the current working directory
            self.run_command(self.get_command('untargz_everything') % (tmpdir))
            self.cleaner.remove_soscleaner_archive()

        # tar all archive folders
        self.run_command(self.get_command('tar_all_dirs') % (tmpdir))

        # remove all archive folders
        self.run_command(self.get_command('remove_all_dirs') % (tmpdir))

        # GZ up the whole folder
        tar_command = self.get_command('compress_archives') % (tmpdir, tmpgz)
        execution = int(self.run_command(tar_command))
        if execution == 0:
            self.logger.vlog(self.language.get_language("Successfully compressed all archives to %s") % (tmpgz))
            return True
        else:
            self.logger.vlog(self.language.get_language("There was an error compressing the archives to %s") % (tmpgz))
            return False

    # Upload archives to Red Hat
    def upload_archives(self, tar_path, thetime):
        conn = connection.InsightsConnection(self.get_uploader_conf())
        try:
            resp = conn.upload_archive(tar_path, thetime, cluster=self.get_cluster_id())
            if resp.status_code == 201:
                self.logger.vlog(self.language.get_language("Successfully uploaded %s to Red Hat.") % (tar_path))
                return True
            else:
                self.logger.vlog("=========================")
                self.logger.vlog(self.language.get_language("ARCHIVE UPLOAD UNSUCCESSFUL"))
                self.logger.vlog(self.language.get_language("There was an issue uploading %s to Red Hat.") % (tar_path))
                self.logger.vlog(self.language.get_language("API response code: %s") % (resp.status_code))
                try:
                    j = resp.json()
                    if j.get("internal_error", {}).get("reference"):
                        self.logger.vlog(self.language.get_language("Failure reference key: %s") %
                                         (str(j["internal_error"]["reference"])))
                except ValueError:
                    pass
                self.logger.vlog(self.language.get_language("Keeping archive saved at %s") % (tar_path))
                self.logger.vlog("=========================")
                return False
        except:
            self.logger.vlog("=========================")
            self.logger.vlog(self.language.get_language("ARCHIVE UPLOAD UNSUCCESSFUL"))
            self.logger.vlog(self.language.get_language("Failed to connect to Red Hat!"))
            self.logger.vlog(self.language.get_language("Keeping archive saved at %s") % (tar_path))
            self.logger.vlog("=========================")
            return False

    #################
    # JSON #
    ###############

    # serialize date/time stuff
    def serialize_default(self, obj):
        if isinstance(obj, datetime):
            return obj.strftime('%Y-%m-%dT%H:%M:%SZ')
        elif isinstance(obj, date):
            return obj.strftime('%Y-%m-%d')
        else:
            return str(obj)

    ###################
    # INSIGHTS #
    #################

    # Get the uploader conf
    def get_uploader_conf(self):
        return self.uploader_conf

    # Get the insights uploader.json and sig config
    def get_insights_config(self):
        conn = InsightsConnection(self.get_uploader_conf())
        branch_info = conn.branch_info()
        rules_config = InsightsConfig(self.get_uploader_conf(), conn)
        collection_rules = rules_config.get_collection_rules(raw=True)
        sig = rules_config.fetch_gpg()
        coord_config = {"sig": sig, "uploader.json": collection_rules, "branch_info": branch_info}
        return coord_config

    ########################
    # CLUSTER STUFF #
    ######################

    # Get the cluster ID
    def get_cluster_id(self):
        if not os.path.exists(self.constants.cluster_id_file):
            return self.generate_cluster_id()
        else:
            f = open(self.constants.cluster_id_file, 'r')
            return f.read()

    # Generate the cluster ID
    def generate_cluster_id(self):
        f = open(self.constants.cluster_id_file, 'wb')
        cluster_id = str(uuid.uuid4())
        f.write(cluster_id)
        f.close()
        return cluster_id
