#!/usr/bin/env python

import paramiko
import glob
import uuid
import json
import os


class RhospCoordinatorNode():

    # Containers
    utilities = None
    logger = None
    constants = None
    ssh_client = None
    director = None
    commands = None
    config = None
    language = None

    # Constructor
    def __init__(self, **kwargs):
        self.constants = kwargs['constants']
        self.config = kwargs['config']
        self.logger = kwargs['logger']
        self.utilities = kwargs['utilities']
        self.ssh_client = kwargs['ssh_client']
        self.director = kwargs['director']
        self.language = kwargs['language']

    # Test if a node is alive
    def test_alive(self, node):
        ping_test = int(self.utilities.run_command(self.utilities.get_command('ping_command') % (str(node))))
        if ping_test == 0:
            self.logger.vlog(self.language.get_language("Node %s is alive.") % (node))
            return True
        else:
            self.logger.vlog(self.language.get_language("Node %s is not alive.") % (node))
            return False

    # Test if a node can be SSHd into
    def test_ssh(self, node):
        ssh_conn = self.ssh_client.get_ssh_client_connection(node)
        if ssh_conn:
            self.logger.vlog(self.language.get_language("Node %s SSH successful.") % (node))
            return True
        else:
            self.logger.vlog(self.language.get_language("Node %s SSH was not successful.") % (node))
            return False

    # Install/Copy the insights RPM on the node in /var/tmp
    # Untar insights on the node
    # This assumes that we have tested and have a valid connection with the specified node
    def install_node(self, ip, options=None):
        # check if the local director installation directory/rpm/gz exists
        local_director_installation_exists = self.director.check_director_installation()
        if not local_director_installation_exists:
            self.director.install_director(options)

        # copy the installation directory/rpm/gz over to be run on remote node
        absolute_insights_path = glob.glob(self.config.get_config_var('local_director_installation'))[0]
        copy_return = self.copy_file_to_node(ip, absolute_insights_path, self.constants.node_installation_gz)
        if not copy_return:
            self.logger.vlog(self.language.get_language("There was an issue copying Insights on %s") % (ip))
            return False
        else:
            self.logger.vlog(self.language.get_language("Successfully copied Insights on %s") % (ip))

        # untar the scpd gz on the node
        gz_return = self.untar_node(ip, options)
        if gz_return:
            self.logger.vlog(self.language.get_language("Successfully untarred Insights on %s") % (ip))
            return True
        else:
            self.logger.vlog(self.language.get_language("There was an issue untarring Insights on %s") % (ip))
            return False

    # Copy the files to the node
    def copy_file_to_node(self, node, source, destination):
        try:
            t = paramiko.Transport((str(node), int(self.config.get_config_var('ssh_port'))))
            rsa_key = paramiko.RSAKey.from_private_key_file(self.config.get_config_var('ssh_key'))
            t.connect(username=self.config.get_config_var('ssh_user'), pkey=rsa_key)
            sftp = paramiko.SFTPClient.from_transport(t)
            sftp_put = sftp.put(source, destination)
            return True
        except Exception as inst:
            return False

    # Untar the insights installation on the node
    def untar_node(self, node, options):
        ssh_command = self.utilities.get_command('untar_node') % (self.constants.node_installation_dir, self.constants.node_installation_dir, self.constants.node_installation_gz)
        ssh_execution = int(self.ssh_client.run_command_on_client(ssh_command, node)['stdout'].read())

        if ssh_execution == 0:
            return True
        else:
            return False

    # Run the insights RPM on the node in /tmp and get STDOUT
    # This assumes that we have tested and have a valid connection with the specified node
    # This assumes we have copied over the RPM into /tmp
    def run_node(self, ip, node_uuid, node_name):

        # for good measures should we check if
        # the remote node installation has been scpd over before running?
        # or should we assume?
        remote_node_installation = self.check_node_installation(ip)

        # if the installation exists then continue
        if remote_node_installation:

            self.logger.vlog(self.language.get_language("Installation exists on RHOSP Node %s(%s), running Insights.") % (node_name, ip))

            # Get the insights config, contains uploader.json and sig
            # config['sig'], config['uploader.json']
            config = self.utilities.get_insights_config()
            config['machine-id'] = node_uuid
            config = json.dumps(config)

            # Write the temporary configuration file on the node
            insights_config_file_name = "/var/tmp/%s" % (node_uuid)
            insights_config_file = open(insights_config_file_name, "wb")
            insights_config_file.write(config)
            insights_config_file.close()
            self.copy_file_to_node(ip, insights_config_file_name, self.constants.insights_config_file)
            os.remove(insights_config_file_name)

            # Run Insights
            insights_command = self.utilities.get_command('insights_execution_command') % (self.constants.insights_config_file)
            command_to_run = "cd %s && %s" % (self.constants.node_installation_dir, insights_command)
            ssh_execution = self.ssh_client.run_command_on_client(command_to_run, ip)
            insights_stdout = ssh_execution['stdout'].read()
            return insights_stdout

        else:
            self.logger.vlog(self.language.get_language("Insights could not be found on the RHOSP node %s(%s).") % (node_name, ip))
            return False

    # Clean up the nodes /tmp from the insights execution
    def clean_node(self, node):
        # clean up the remote node installation directodry/gz
        self.logger.vlog(self.language.get_language("Cleaning node %s") % (node))

        # TODO: Should this be in the commands.py file?
        node_command_to_run = "rm -rf %s %s %s ; echo $?" % (self.constants.node_installation_dir, self.constants.node_installation_gz, self.constants.insights_config_file)
        ssh_execution = int(self.ssh_client.run_command_on_client(node_command_to_run, node)['stdout'].read())

        if ssh_execution == 0:
            self.logger.vlog(self.language.get_language("Node %s successfully cleaned.") % (node))
            return True
        else:
            self.logger.vlog(self.language.get_language("There was an issue cleaning %s.") % (node))
            return False

    # Check if the Node has received the correct Insights
    def check_node_installation(self, node):
        ssh_command = self.utilities.get_command('check_node_installation') % (self.constants.node_installation_gz, self.constants.node_installation_dir)
        ssh_execution = int(self.ssh_client.run_command_on_client(ssh_command, node)['stdout'].read())

        self.logger.vlog(self.language.get_language("Checking if Insights is installed on node %s") % (node))

        if ssh_execution == 0:
            self.logger.vlog(self.language.get_language("Insights is installed on node %s") % (node))
            return True
        else:
            self.logger.vlog(self.language.get_language("Insights installation not found on node %s") % (node))
            return False

    # Generate a machine-id for a node
    # Uses the RHOSP Director UUID and the Nodes RHOSP UUID from the API
    # This gives us a deterministic UUID
    def generate_machine_id(self, director_uuid, node_uuid):
        return str(uuid.uuid5(uuid.UUID(director_uuid), str(node_uuid)))
