"""
Imports
"""

import sys
import time

"""
Run the platform module
Main execution function
Called from the main coordinator
"""


def run(**kwargs):
    options = kwargs['options']
    constants = kwargs['constants']
    utilities = kwargs['utilities']
    logger = kwargs['logger']
    insights_uploader_conf = kwargs['uploader_conf']
    config = kwargs['config']
    language = kwargs['language']
    director = kwargs['director']
    ssh_client = kwargs['ssh_client']
    versions = kwargs['versions']

    logger.vlog("Running RHOSP Insights Coordinator")
    versions.get_versions()
    starttime = time.time()

    # Check if we are only testing the director api SDK
    # TODO: Make this verbose and spit out the API Credentials used to verify the connection
    if options.test_api:
        api_status = director.test_api()
        logger.vlog(language.get_language("API Connection Status: %s") % (api_status))
        sys.exit(language.get_language("We are only testing the RHOSP Director API. The coordinator is now exiting."))

    # Parse the blacklist (if present)
    blacklist = []
    if options.blacklist and options.blacklist != '' and len(options.blacklist) > 0:
        blacklist = options.blacklist.split(',')
        blacklist = blacklist if len(blacklist) > 0 else []
    # Clean the blacklist keywords
    # Replace asterisks with (.*)
    # Create anchors
    if len(blacklist) > 0:
        cleaned_blacklist = []
        for keyword in blacklist:
            cleaned_blacklist.append(('^%s$') % (keyword.replace('*', '(.*)')))
        blacklist = cleaned_blacklist
        logger.vlog('Blacklist parsed: %s' % (blacklist))
    else:
        logger.vlog('No blacklist found.')
    director.setup_blacklist(blacklist)

    # Get all nodes/ips
    logger.vlog(language.get_language("Preparing to gather nodes and services from the director..."))
    if options.hosts_only:
        logger.set_verbosity(True)
    node_list = director.get_node_list()
    service_list = director.get_service_list()
    director.print_node_list(node_list)
    director.print_service_list(service_list)
    if options.hosts_only:
        sys.exit(language.get_language("We are only testing if the nodes are alive and can be remotely accessed. Coordinator is exiting."))

    # Setup Director Options
    director_options = constants.api_options
    # TODO: Should probably move this logic into the director class
    if options.api_options:
        if options.api_options == 'none':
            logger.vlog(language.get_language("We are not collecting any metadata from the RHOSP Director API, 'none' was passed in."))
            director_options = []
        elif options.api_options == 'all':
            logger.vlog(language.get_language("We are collecting all metadata from the RHOSP Director API. 'all' was passed in."))
            director_options = constants.api_options.split(',')
        else:
            director_options = options.api_options.split(',')
            logger.vlog(language.get_language("We are collecting specific metadata from the RHOSP Director API."))
            logger.vlog(director_options)

    # Collect the specific RHOSP Director API metadata
    director.collect_director_information(director_options)

    # Run Insights on the Director
    # We run Insights on the Director first so that the Collection Rules are automatically updated
    # This ensures that each subsequent configuration file that is passed to the individual nodes contains the updated rules
    director.run_insights_on_director()

    # FOR ALL NODES
    # 1) ssh into each node/ip and install latest client in /tmp, this will be removed once we have the insights RPM on the OSP image
    # 2) run latest client on each node in /tmp, will be modified to run native installed insights rpm once we have it on the images
    # 3) get stdout from client
    # 4) clean /tmp
    stdout = ""
    total_nodes = len(node_list)
    for index, node in enumerate(node_list):
        logger.vlog("========================================")
        logger.vlog("Processing node %s out of %s: %s, %s, %s" % (index + 1, total_nodes, node['id'], node['ip'], node['name']))
        node_blacklisted = node['blacklisted']
        if node['is_alive'] and node['can_ssh'] and not node_blacklisted:
            director.get_node().install_node(node['ip'], options)
            stdout = director.get_node().run_node(node['ip'], node['id'], node['name'])
            utilities.write_archive(stdout)
            director.get_node().clean_node(node['ip'])
        else:
            if not node['is_alive']:
                logger.vlog('Cannot execute Insights on Node %s, %s, %s as it is not alive.' % (node['id'], node['ip'], node['name']))
            if not node['can_ssh']:
                logger.vlog('Cannot execute Insights on Node %s, %s, %s as the Coordinator does not have valid SSH access.' % (node['id'], node['ip'], node['name']))
            if node_blacklisted:
                logger.vlog('Node %s, %s, %s has been blacklisted via --blacklist or the Coordinator configuration files. Insights will not be run for this Node.' % (node['id'], node['ip'], node['name']))
        logger.vlog("Finished processing node.")
        logger.vlog("========================================")

    # Tar everything up and clean
    utilities.compress_archives()

    # Get execution time elapsed
    thetime = time.time() - starttime

    # Upload to Red Hat if we are not only archiving
    upload_success = False
    logger.vlog('Archive Only: %s' % (options.archive_only))
    if options.archive_only:
        logger.vlog(language.get_language("Archive Only was set. Not uploading archives to Red Hat."))
    else:
        logger.vlog(language.get_language("Archive Only flag was not set. Uploading archives to Red Hat."))
        upload_success = utilities.upload_archives(utilities.get_tmpgz(), thetime)

    # Clean the temporary files if we are not only archiving
    logger.vlog('Upload Success: %s' % (upload_success))
    if options.archive_only or not upload_success:
        logger.vlog(language.get_language("Archive saved to %s.") % (utilities.get_tmpgz()))
    else:
        logger.vlog(language.get_language("Cleaning tmps."))
        utilities.clean_tmps()

"""
MAIN EXECUTION
"""
if __name__ == "__main__":
    import doctest
    doctest.testmod()
